/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDItemEditor.h"



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDItemEditor::VDItemEditor() {
    // TODO
    itemSprite = NULL;
    (void) strcpy_s(currentItemTypeSelection, "");
    armorLocationStatic = NULL;
    armorLocationComboBox = NULL;
    armorProtectionStatic = NULL;
    armorProtectionComboBox = NULL;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDItemEditor::~VDItemEditor() {
    FreeMemory();
};



void VDItemEditor::FreeMemory() {
    if (itemSprite != NULL) {
        itemSprite->FreeTexture();
        delete itemSprite;
        itemSprite = NULL;
    };
};



/* **********************************************************************************************************************
   Load
   ----
   Creates a new Item Editor window.
   #param  instance      The instance of the application.
   #param  parentWindow  Handle to the main window (to use for the starting x, y position)
   #param  className     The name of the window class.
   #return  Handle to the window.
   **********************************************************************************************************************/

HWND VDItemEditor::Load(HINSTANCE instance, HWND parentWindow, LPCSTR className) {
    RECT windowDimensions;
    (void) GetWindowRect(parentWindow, &windowDimensions);
    window = CreateWindowEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW,      // extended window styles
                            className,                             // class name
                            "Item Editor",                         // window name
                            WS_CAPTION | WS_SYSMENU | WS_VISIBLE,  // window styles
                            windowDimensions.left + 100,           // x coordinate (0 = left)
                            windowDimensions.top + 100,            // y coordinate (0 = top)
                            440,                                   // width
                            530,                                   // height
                            NULL,                                  // parent/owner window
                            NULL,                                  // menu
                            instance,                              // instance
                            NULL);                                 // used for CREATESTRUCT structure
    if (window == NULL) {
        FreeMemory();
        VDError("Failed to create the window.", __FILE__, __LINE__);
    };

    // Items
    HWND itemListStatic = CreateWindow("Static", "Items:", WS_CHILD | WS_VISIBLE, 10, 10, 40, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_NPC_LIST, instance, NULL);
    if (itemListStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    itemListBox = CreateWindow("ListBox", NULL, WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 30, 100, 100, window, (HMENU) VD_NPC_EDITOR_NPC_LIST, instance, NULL);
    if (itemListBox == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) strcpy_s(currentItemSelection, "");
    HWND npcNameStatic = CreateWindow("Static", "Name:", WS_CHILD | WS_VISIBLE, 120, 30, 50, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_NAME, instance, NULL);
    if (npcNameStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    nameEditBox = CreateWindow("Edit", "", WS_CHILD | WS_BORDER | WS_VISIBLE, 180, 30, 140, 18, window, (HMENU) VD_NPC_EDITOR_NAME, instance, NULL);
    if (nameEditBox == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadSpriteButton = CreateWindow("Button", "Load Image", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 120, 60, 80, 20, window, (HMENU) VD_NPC_EDITOR_LOAD_SPRITE, instance, NULL);
    if (loadSpriteButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND spriteBorderStatic = CreateWindow("Static", "", WS_CHILD | WS_VISIBLE | WS_BORDER, 120, 85, 48, 48, window, (HMENU) VD_NPC_EDITOR_STATIC_SPRITE_BORDER, instance, NULL);
    if (spriteBorderStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    FreeMemory();
    (void) strcpy_s(itemSpriteImageLocation, "");
    itemSpriteDimensions.left = 128;
    itemSpriteDimensions.top = 93;
    itemSpriteDimensions.right = 160;
    itemSpriteDimensions.bottom = 125;
    shiftSpriteUp = CreateWindow("Button", "/\\", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 168, 85, 20, 20, window, (HMENU) VD_NPC_EDITOR_SHIFT_NPC_UP, instance, NULL);
    if (shiftSpriteUp == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftSpriteDown = CreateWindow("Button", "\\/", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 168, 113, 20, 20, window, (HMENU) VD_NPC_EDITOR_SHIFT_NPC_DOWN, instance, NULL);
    if (shiftSpriteDown == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftSpriteLeft = CreateWindow("Button", "<", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 120, 133, 20, 20, window, (HMENU) VD_NPC_EDITOR_PREVIOUS_ANIMATION, instance, NULL);
    if (shiftSpriteLeft == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftSpriteRight = CreateWindow("Button", ">", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 148, 133, 20, 20, window, (HMENU) VD_NPC_EDITOR_NEXT_ANIMATION, instance, NULL);
    if (shiftSpriteRight == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    /*removeItemButton = CreateWindow("Button", "Remove Item", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 133, 100, 20, window, (HMENU) VD_NPC_EDITOR_REMOVE_NPC, instance, NULL);
    if (removeItemButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    addItemButton = CreateWindow("Button", "Save Item", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 220, 100, 80, 20, window, (HMENU) VD_NPC_EDITOR_SAVE_NPC, instance, NULL);
    if (addItemButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };*/

    // item type
    HWND typeStatic = CreateWindow("Static", "Type:", WS_CHILD | WS_VISIBLE, 10, 135, 100, 18, window, (HMENU) 666, instance, NULL);
    itemTypeComboBox = CreateWindow("ComboBox", "", WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST | WS_VSCROLL, 10, 155, 200, 100, window, (HMENU) 6667, instance, NULL);
    // TODO: !!! Have to have a size of the item types somehow...
    for (int n = 0; n < VDItem::NUMITEMTYPES; n++) {
        (void) SendMessage(itemTypeComboBox, CB_ADDSTRING, 0, (LPARAM) VDItem::ITEMTYPES[n]);
    };

    // save items
    saveChangesButton = CreateWindow("Button", "SAVE CHANGES", WS_CHILD | BS_PUSHBUTTON | WS_BORDER | WS_VISIBLE, 115, 460, 200, 40, window, (HMENU) VD_NPC_EDITOR_SAVE_CHANGES, instance, NULL);
    if (saveChangesButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    return window;
};



/* **********************************************************************************************************************
   ProcessMessages
   ---------------
   Processes messages sent by and received from the Item Editor.
   #param TODO
   **********************************************************************************************************************/

void VDItemEditor::ProcessMessages(HINSTANCE instance, LPDIRECT3DDEVICE9 direct3DDevice, VDInput* input, vector<VDItem>* items) {
    // set the input to the Item Editor
    if (input->GetCurrentWindow() != window) {
        input->ChangeInputWindow(window);
    };

    // populate the item list if it's empty
    if (SendMessage(itemListBox, LB_GETCOUNT, 0, 0) == 0) {
        for (size_t n = 0; n < items->size(); n++) {
            (void) SendMessage(itemListBox, LB_ADDSTRING, 0, (LPARAM) items->at(n).GetName());
        };
    };

    // user has an item type selected
    /////////////////////////////////

    if (SendMessage(itemTypeComboBox, CB_GETDROPPEDSTATE, 0, 0) == FALSE) {
        int currentItemTypeIndex = (int) SendMessage(itemTypeComboBox, CB_GETCURSEL, 0, 0);
        if (currentItemTypeIndex != LB_ERR) {
            // user selects a different item type
            /////////////////////////////////////

            if (strcmp(currentItemTypeSelection, VDItem::ITEMTYPES[currentItemTypeIndex]) != 0) {
                // copy the current selection to previous
                (void) strcpy_s(currentItemTypeSelection, VDItem::ITEMTYPES[currentItemTypeIndex]);

                // clear the data
                if (armorLocationStatic != NULL) {
                    (void) DestroyWindow(armorLocationStatic);
                };
                if (armorLocationComboBox != NULL) {
                    (void) DestroyWindow(armorLocationComboBox);
                };
                if (armorProtectionStatic != NULL) {
                    (void) DestroyWindow(armorProtectionStatic);
                };
                if (armorProtectionComboBox != NULL) {
                    (void) DestroyWindow(armorProtectionComboBox);
                };

                // TODO: check the item type and load up the following

                // armor
                if (currentItemTypeIndex >= 0 && currentItemTypeIndex <= 3) {
                    // armor location
                    armorLocationStatic = CreateWindow("Static", "Armor Location:", WS_CHILD | WS_VISIBLE, 10, 185, 110, 18, window, (HMENU) 6662, instance, NULL);
                    armorLocationComboBox = CreateWindow("ComboBox", "", WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST | WS_VSCROLL, 10, 205, 200, 100, window, (HMENU) 2444, instance, NULL);
                    for (int n = 0; n < VDItem::NUMLOCATIONS; n++) {
                        (void) SendMessage(armorLocationComboBox, CB_ADDSTRING, 0, (LPARAM) VDItem::LOCATIONS[n]);
                    };

                    // armor protection
                    
                    armorProtectionStatic = CreateWindow("Static", "Armor Protection:", WS_CHILD | WS_VISIBLE, 10, 235, 115, 18, window, (HMENU) 2311, instance, NULL);
                    armorProtectionComboBox = CreateWindow("ComboBox", "", WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST | WS_VSCROLL, 10, 255, 200, 100, window, (HMENU) 1234, instance, NULL);
                    // TODO: loop through the list of armor protections, populate the combo box
                };

                // item - armor hindrance
                // item - weapon damage
            };
        };
    };



    // Save Changes button
    //////////////////////

    if (SendMessage(saveChangesButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        // save the NPCs
        WriteItems(items);

        // lets other editors know we've changed NPC's
        reloadItems = TRUE;

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };
};



BOOL VDItemEditor::AddItem(vector<VDItem>* items, LPCSTR name, LPCSTR imageFileLocation) {
    // see if we're trying to add an NPC that already exists
    for (unsigned int n = 0; n < items->size(); n++) {
        if (strcmp(name, items->at(n).GetName()) == 0) {
            items->at(n).SetImageFileLocation(imageFileLocation);
            // TODO: set item data like dmg, absorption, speed, etc.
            return FALSE;
        };
    };

    // add the NPC
    VDItem item;
    item.SetName(name);
    item.SetImageFileLocation(imageFileLocation);
    items->push_back(item);

    return TRUE;
};



/* **********************************************************************************************************************
   WriteItems
   ----------
   #param  items  Pointer to the list of items.
   Saves the items to hard disk.
   **********************************************************************************************************************/

void VDItemEditor::WriteItems(vector<VDItem>* items) {
    HANDLE file = NULL;
    BOOL returnValue = FALSE;
    DWORD numberOfBytesWritten = 0;
    size_t numberOfItems = 0;

    // open the file
    file = CreateFile("Data/Items.vd", GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if (file == INVALID_HANDLE_VALUE) {
        FreeMemory();
        VDError("Failed to create the Items.vd file.", __FILE__, __LINE__);
    };

    // make sure we let it know how many item elements we're adding
    numberOfItems = items->size();
    returnValue = WriteFile(file, &numberOfItems, sizeof(numberOfItems), &numberOfBytesWritten, NULL);
    if (returnValue == FALSE) {
        FreeMemory();
        VDError("Failed to write item data to file.", __FILE__, __LINE__);
    };

    // write all the items
    for (size_t n = 0; n < numberOfItems; n++) {
        items->at(n).WriteToFile(file);
        if (returnValue == FALSE) {
            FreeMemory();
            VDError("Failed to write item data to file.", __FILE__, __LINE__);
        };
    };

    // close the file
    returnValue = CloseHandle(file);
    if (returnValue == FALSE) {
        FreeMemory();
        VDError("Failed to close the Items.vd file.", __FILE__, __LINE__);
    };
};



VDItem* VDItemEditor::GetVDItem(vector<VDItem>* items, LPCSTR name) {
    for (unsigned int n = 0; n < items->size(); n++) {
        if (strcmp(name, items->at(n).GetName()) == 0) {
            return &items->at(n);
        };
    };

    return NULL;
};